Package: lisaClust
Type: Package
Title: lisaClust: Clustering of Local Indicators of Spatial Association
Version: 1.18.0
Authors@R: c(
    person("Ellis", "Patrick", , "ellis.patrick@sydney.edu.au", role = c("aut", "cre")),
    person("Nicolas", "Canete", , "nicolas.canete@sydney.edu.au", role = "aut"),
    person("Nicholas", "Robertson", , "nicholas.robertson@sydney.edu.au", role = "ctb"),
    person("Alex", "Qin", , "alex.qin@sydney.edu.au", role = "ctb"),
    person("Shreya", "Rao", "shreya.rajeshrao@sydney.edu.au", role = "ctb")
  )
Description: lisaClust provides a series of functions to identify and visualise 
    regions of tissue where spatial associations between cell-types is similar.
    This package can be used to provide a high-level summary of cell-type 
    colocalization in multiplexed imaging data that has been segmented at a 
    single-cell resolution.
License: GPL (>=2)
biocViews: SingleCell, CellBasedAssays, Spatial
Encoding: UTF-8
Depends: R (>= 4.1.0)
VignetteBuilder: knitr
BugReports: https://github.com/ellispatrick/lisaClust/issues
URL: https://ellispatrick.github.io/lisaClust/,
        https://github.com/ellispatrick/lisaClust
Imports: ggplot2, class, concaveman, grid, BiocParallel,
        spatstat.explore, spatstat.geom, BiocGenerics, S4Vectors,
        methods, spicyR, purrr, stats, data.table, dplyr, tidyr,
        SingleCellExperiment, SpatialExperiment, SummarizedExperiment,
        pheatmap, spatstat.random, lifecycle, simpleSeg, rlang,
Suggests: SpatialDatasets, BiocStyle, knitr, rmarkdown, testthat (>=
        3.0.0)
RoxygenNote: 7.3.2
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/lisaClust
git_branch: RELEASE_3_22
git_last_commit: 8ecd502
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 04:49:12 UTC; biocbuild
Author: Ellis Patrick [aut, cre],
  Nicolas Canete [aut],
  Nicholas Robertson [ctb],
  Alex Qin [ctb],
  Shreya shreya.rajeshrao@sydney.edu.au Rao [ctb]
Maintainer: Ellis Patrick <ellis.patrick@sydney.edu.au>
